@@ -18,7 +18,7 @@ from .utils import (
     is_tooled,
 )
 
-_valid_hashvars = ("#enter", "#value", "#yield")
+_valid_hashvars = ("#enter", "#error", "#exit", "#receive", "#value", "#yield")
 
 
 class SelectorError(Exception):
@@ -325,6 +325,11 @@ class Call(Selector):
                             f"No variable in `{func}` has the category `{x.category}`"
                         )
 
+                elif x.name.startswith("#loop_") or x.name.startswith(
+                    "#endloop_"
+                ):
+                    pass
+
                 elif x.name.startswith("#"):
                     if x.name not in _valid_hashvars:
                         problems.append(
@@ -365,9 +370,9 @@ parser = opparse.Parser(
             ",": opparse.rassoc(10),
             ("", ">", ">>"): opparse.rassoc(100),
             ("=", "~"): opparse.lassoc(120),
-            ("!", "!!"): opparse.lassoc(150),
             ":": opparse.lassoc(300),
             "as": opparse.rassoc(350),
+            ("!", "!!"): opparse.lassoc(375),
             "$": opparse.lassoc(400),
             ("(", "[", "{", "[["): opparse.obrack(200),
             (")", "]", "}", "]]"): opparse.cbrack(500),
@@ -481,13 +486,13 @@ def make_focus(node, _, element, context):
 def make_double_focus(node, _, element, context):
     element = evaluate(element, context=context)
     assert isinstance(element, Element)
-    return element.clone(tags=frozenset(element.tags | {2}))
+    return element.clone(tags=frozenset({2}))
 
 
 @evaluate.register_action("_ $ X")
 def make_dollar(node, _, name, context):
     name = evaluate(name, context=context)
-    return Element(name=None, category=None, capture=name.name)
+    return Element(name=None, category=None, capture=name.name, tags=name.tags)
 
 
 @evaluate.register_action("X ( _ ) _")
@@ -518,13 +523,13 @@ def make_as(node, element, name, context):
     element = evaluate(element, context=context)
     name = evaluate(name, context=context)
     if isinstance(element, Element):
-        return element.clone(capture=name.name)
+        return element.clone(capture=name.name, tags=element.tags | name.tags)
     else:
         focus = context == "root"
         new_capture = Element(
             name="#value",
             capture=name.name,
-            tags=frozenset({1}) if focus else frozenset(),
+            tags=name.tags or (frozenset({1}) if focus else frozenset()),
         )
         return element.clone(captures=element.captures + (new_capture,))
 
@@ -756,6 +761,8 @@ class MatchFunction:
 def _dig(fn):
     while hasattr(fn, "__wrapped__") and not is_tooled(fn):
         fn = fn.__wrapped__
+    if isinstance(fn, property):
+        return _dig(fn.fget)
     return fn
 
 
