@@ -15,7 +15,7 @@ from textwrap import dedent
 from types import TracebackType
 
 from .selector import Element, check_element
-from .tags import get_tags
+from .tags import enter_tag, exit_tag, get_tags
 from .utils import ABSENT, DictPile
 
 _IDX = count()
@@ -186,6 +186,15 @@ class ExternalVariableCollector(NodeVisitor):
         self.assigned.add(node.arg)
 
 
+class SimpleVariableCollector(NodeVisitor):
+    def __init__(self, tree):
+        self.vars = set()
+        self.visit(tree)
+
+    def visit_Name(self, node):
+        self.vars.add(node.id)
+
+
 class PteraTransformer(NodeTransformer):
     """Transform the AST of a function to instrument it with ptera.
 
@@ -204,7 +213,7 @@ class PteraTransformer(NodeTransformer):
         for ext in self.external:
             self.provenance[ext] = "external"
         self.annotated = {}
-        self.evalcache = {}
+        self.evalcache = {None: ABSENT}
         self.linenos = {}
         self.defaults = {}
         self.lib = lib
@@ -213,7 +222,7 @@ class PteraTransformer(NodeTransformer):
         self.to_instrument = to_instrument
         self.result = self.visit_FunctionDef(tree, root=True)
 
-    def should_instrument(self, varname, ann):
+    def should_instrument(self, varname, ann=None):
         evaluated_ann = self._evaluate(ann)
         if any(
             check_element(el, varname, evaluated_ann)
@@ -244,6 +253,7 @@ class PteraTransformer(NodeTransformer):
     def _evaluate(self, node):
         if node in self.evalcache:
             return self.evalcache[node]
+        ast.fix_missing_locations(node)
         try:
             result = eval(
                 compile(
@@ -254,7 +264,7 @@ class PteraTransformer(NodeTransformer):
                 self.globals,
                 self.globals,
             )
-        except Exception:
+        except Exception:  # pragma: no cover
             result = ABSENT
         self.evalcache[node] = result
         return result
@@ -293,6 +303,61 @@ class PteraTransformer(NodeTransformer):
             keywords=[],
         )
 
+    def standalone_interaction(self, *args):
+        itn = self._interact(*args)
+        if isinstance(itn, ast.Constant):  # pragma: no cover
+            return []
+        else:
+            return [ast.Expr(itn)]
+
+    def delimit(self, body, enter, error, exit, enter_tag=None, exit_tag=None):
+        enter = [x for x in enter if self.should_instrument(x, enter_tag)]
+        error = [x for x in error if self.should_instrument(x)]
+        exit = [x for x in exit if self.should_instrument(x, exit_tag)]
+
+        enter_stmts = [
+            self.standalone_interaction(sym, None, enter_tag, True, False)
+            for sym in enter
+        ]
+        body = reduce(list.__add__, [*enter_stmts, body])
+
+        if not error and not exit:
+            return body
+
+        exit_stmts = [
+            self.standalone_interaction(sym, None, exit_tag, True, False)
+            for sym in exit
+        ]
+        finalbody = reduce(list.__add__, exit_stmts, [])
+
+        handlers = [
+            ast.ExceptHandler(
+                type=ast.Name(id="BaseException", ctx=ast.Load()),
+                name="#error",
+                body=[
+                    *self.standalone_interaction(
+                        sym,
+                        None,
+                        None,
+                        ast.Name(id="#error", ctx=ast.Load()),
+                        False,
+                    ),
+                    ast.Raise(),
+                ],
+            )
+            for sym in error
+        ]
+
+        trycatch = ast.Try(
+            body=body,
+            handlers=handlers,
+            orelse=[],
+            finalbody=finalbody,
+        )
+        body = [trycatch]
+
+        return body
+
     def make_interaction(self, target, ann, value, orig=None, expression=False):
         """Create code for setting the value of a variable."""
         if ann and isinstance(target, ast.Name):
@@ -431,9 +496,7 @@ class PteraTransformer(NodeTransformer):
         if not root:
             return node
 
-        wrapped_body = []
-
-        new_body = [ast.Expr(self._interact("#enter", None, None, True, False))]
+        new_body = []
 
         for external in sorted(self.external):
             new_body.extend(
@@ -461,9 +524,12 @@ class PteraTransformer(NodeTransformer):
 
         new_body += self.generate_interactions(node.args)
 
+        wrapped_body = []
+
         body = node.body
         first = body[0]
         if isinstance(first, ast.Expr):
+            # Pull the docstring into wrapped_body
             v = first.value
             if (
                 isinstance(v, ast.Str)
@@ -474,6 +540,14 @@ class PteraTransformer(NodeTransformer):
                 body = body[1:]
 
         new_body += self.visit_body(node.body)
+        new_body = self.delimit(
+            new_body,
+            ["#enter"],
+            ["#error"],
+            ["#exit"],
+            enter_tag=self._get("enter_tag"),
+            exit_tag=self._get("exit_tag"),
+        )
 
         wrapped_body.append(
             ast.With(
@@ -507,6 +581,15 @@ class PteraTransformer(NodeTransformer):
         new_body = self.generate_interactions(node.target)
         new_body.extend(self.visit_body(node.body))
 
+        svc = SimpleVariableCollector(node.target)
+
+        new_body = self.delimit(
+            new_body,
+            [f"#loop_{v}" for v in svc.vars],
+            [],
+            [f"#endloop_{v}" for v in svc.vars],
+        )
+
         return ast.copy_location(
             ast.For(
                 target=node.target,
@@ -529,7 +612,7 @@ class PteraTransformer(NodeTransformer):
         return ast.copy_location(
             ast.ExceptHandler(
                 name=node.name,
-                type=self.visit(node.type),
+                type=node.type and self.visit(node.type),
                 body=new_body,
             ),
             node,
@@ -574,8 +657,8 @@ class PteraTransformer(NodeTransformer):
         After:
             x = _ptera_interact('x', None, y + z)
         """
-        (target,) = node.targets
-        if isinstance(target, ast.Tuple):
+
+        def _decompose(targets, transform):
             var_all = _gensym()
             ass_all = ast.copy_location(
                 ast.Assign(
@@ -585,23 +668,53 @@ class PteraTransformer(NodeTransformer):
                 node,
             )
             accum = [ass_all]
-            for i, tgt in enumerate(target.elts):
+            for i, tgt in enumerate(targets):
                 accum += self.visit_Assign(
                     ast.copy_location(
                         ast.Assign(
                             targets=[tgt],
-                            value=ast.Subscript(
-                                value=ast.Name(id=var_all, ctx=ast.Load()),
-                                slice=ast.Index(value=ast.Constant(i)),
-                                ctx=ast.Load(),
+                            value=transform(
+                                ast.Name(id=var_all, ctx=ast.Load()), i
                             ),
                         ),
                         node,
                     )
                 )
             return accum
+
+        targets = node.targets
+        if len(targets) > 1:
+            return _decompose(targets, lambda value, i: value)
+
+        elif isinstance(targets[0], ast.Tuple):
+            return _decompose(
+                targets[0].elts,
+                lambda value, i: ast.Subscript(
+                    value=value,
+                    slice=ast.Index(value=ast.Constant(i)),
+                    ctx=ast.Load(),
+                ),
+            )
         else:
-            return self.make_interaction(target, None, node.value, orig=node)
+            return self.make_interaction(
+                targets[0], None, node.value, orig=node
+            )
+
+    def visit_AugAssign(self, node):
+        if isinstance(node.target, ast.Name) and self.should_instrument(
+            node.target.id, None
+        ):
+            return [
+                self.generic_visit(node),
+                *self.make_interaction(
+                    node.target,
+                    None,
+                    ast.Name(id=node.target.id, ctx=ast.Load()),
+                    orig=node,
+                ),
+            ]
+        else:
+            return self.generic_visit(node)
 
     def visit_Import(self, node):
         """Rewrite an import statement.
@@ -650,11 +763,18 @@ class PteraTransformer(NodeTransformer):
         new_value = self._interact(
             "#yield",
             None,
-            None,
+            self._get("exit_tag"),
             self.visit(node.value or ast.Constant(value=None)),
             True,
         )
-        return ast.copy_location(ast.Yield(value=new_value), node)
+        new_yield = self._interact(
+            "#receive",
+            None,
+            self._get("enter_tag"),
+            ast.Yield(value=new_value),
+            True,
+        )
+        return ast.copy_location(new_yield, node)
 
 
 class _Conformer:
@@ -744,6 +864,39 @@ def _compile(filename, tree, freevars):
     return compile(ast.Module(body=[tree], type_ignores=[]), filename, "exec")
 
 
+def _standard_info():
+    return {
+        "#enter": {
+            "name": "#enter",
+            "annotation": enter_tag,
+            "provenance": "meta",
+            "doc": None,
+            "location": None,
+        },
+        "#exit": {
+            "name": "#exit",
+            "annotation": exit_tag,
+            "provenance": "meta",
+            "doc": None,
+            "location": None,
+        },
+        "#receive": {
+            "name": "#receive",
+            "annotation": enter_tag,
+            "provenance": "meta",
+            "doc": None,
+            "location": None,
+        },
+        "#yield": {
+            "name": "#yield",
+            "annotation": exit_tag,
+            "provenance": "meta",
+            "doc": None,
+            "location": None,
+        },
+    }
+
+
 def transform(fn, proceed, to_instrument=True, set_conformer=True):
     """Return an instrumented version of fn.
 
@@ -838,6 +991,8 @@ def transform(fn, proceed, to_instrument=True, set_conformer=True):
         "get_tags": ("__ptera_get_tags", get_tags),
         "self": (fnsym, None),
         "frame": ("__ptera_frame", None),
+        "enter_tag": ("__ptera_enter_tag", enter_tag),
+        "exit_tag": ("__ptera_exit_tag", exit_tag),
     }
     glb.update(
         {name: value for name, value in lib.values() if value is not None}
@@ -904,6 +1059,7 @@ def transform(fn, proceed, to_instrument=True, set_conformer=True):
         }
         for k in all_vars
     }
+    info.update(_standard_info())
 
     if set_conformer:
         actual_fn._conformer = _Conformer(fn, actual_fn, proceed)
